/**
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information:<br>
 * BitCtrl Systems GmbH<br>
 * Wei&szlig;enfelser Stra&szlig;e 67<br>
 * 04229 Leipzig<br>
 * Phone: +49 341-490670<br>
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.Properties;

import org.junit.Test;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.CommunicationError;
import de.bsvrz.dav.daf.main.ConnectionException;
import de.bsvrz.dav.daf.main.InconsistentLoginException;
import de.bsvrz.dav.daf.main.MissingParameterException;
import de.bsvrz.kex.bwsin.konfig.okstra.Okstra;
import de.bsvrz.kex.bwsin.konfig.okstra.OkstraFeatures;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: TestBetriebskilometer.java 7552 2008-03-18 13:02:22Z gieseler $
 * 
 */
public class TestBetriebskilometer {

	private Okstra odb;
	private ClientDavConnection dav;

	public void initDav() {
		String[] commandArguments = {
				"-datenverteiler=127.0.0.1:8083",
				"-benutzer=Tester",
				"-authentifizierung=C:\\Dokumente und Einstellungen\\Steffen\\Eigene Dateien\\workspace\\DAV-Neu\\Projekte\\SIN-Test\\skripte-dosshell\\passwd",
				"-debugSetLoggerAndLevel=ALL",
				"-debugSetLoggerAndLevel=de.bsvrz.kex.bwsin:ALL",
				"-debugLevelStdErrText=ALL" };
		ArgumentList argumentList = new ArgumentList(commandArguments);
		try {
			String myname = "SIN-TEST";
			Debug.init(myname, argumentList);
			ClientDavParameters parameters = new ClientDavParameters(
					commandArguments);
			parameters.setApplicationName(myname);
			dav = new ClientDavConnection(parameters);
			dav.connect();
			dav.login();
		} catch (NoSuchElementException e) {
			// TODO Automatisch erstellter Catch-Block
			e.printStackTrace();
		} catch (MissingParameterException e) {
			// TODO Automatisch erstellter Catch-Block
			e.printStackTrace();
		} catch (CommunicationError e) {
			// TODO Automatisch erstellter Catch-Block
			e.printStackTrace();
		} catch (ConnectionException e) {
			// TODO Automatisch erstellter Catch-Block
			e.printStackTrace();
		} catch (InconsistentLoginException e) {
			// TODO Automatisch erstellter Catch-Block
			e.printStackTrace();
		}

	}

	public void initOkstra() {
		Properties props = new Properties();
		Map se = new HashMap();

		// props.put("Pfad",
		// "/home/gysi/Documents/BaW/SIN/VRZ_BW_DSK_MIF_1_1/");

		Map<String, Object> m = new HashMap<String, Object>();
		// String
		// URLstr="http://services.interactive-instruments.de/vrzbw/vrz3-wfs.exe";
		String URLstr = "file://C:\\Dokumente und Einstellungen\\Steffen\\Eigene Dateien\\workspace\\wfsclient\\okstrads";
		// String URLstr="file:///home/gysi/workspace/wfsclient/okstrads";
		URL url;
		try {
			url = new URL(URLstr);
		} catch (MalformedURLException e) {
			// TODO Automatisch erstellter Catch-Block
			e.printStackTrace();
			return;
		}
		props.put("SERVER_URL", url);
		props.put("FILESTORE", Boolean.TRUE);

		// props.put(WFSDataStoreFactory.TIMEOUT.key,new Integer(3000000));
		// props.put(WFSDataStoreFactory.PROTOCOL.key, Boolean.FALSE); // false
		// == GET, true == POST
		// props.put(WFSDataStoreFactory.BUFFER_SIZE.key, new Integer(5000000));
		// // false == GET, true == POST

		try {
			odb = new Okstra(props);
		} catch (ImportException e) {
			// TODO Automatisch erstellter Catch-Block
			e.printStackTrace();
		}
	}

	/**
	 * Methode fr
	 * {@link de.bsvrz.kex.bwsin.konfig.BetriebsKilometer#getAoaPunkte(java.lang.String)}
	 * testen
	 */
	// @Test
	public void testGetAoaPunkteString() {
		fail("Noch nicht implementiert");
	}

	/**
	 * Methode fr
	 * {@link de.bsvrz.kex.bwsin.konfig.BetriebsKilometer#getAoaPunkte(java.lang.String, int, int)}
	 * testen
	 */
	@Test
	public void testGetAoaPunkteStringIntInt() {
		initDav();
		initOkstra();
		BetriebsKilometer bk = new BetriebsKilometer();

		// okstra:Ast bentigt okstraProfileVRZ3Netz, dazu zuerst
		// ein vrz3-Feature lesen
		try {
			odb.getFeature(OkstraFeatures.GESCHWINDIGKEIT, "baba");
		} catch (ImportException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		bk.initialisiere(odb);
		List<BetriebsKilometerPunkt> liste = bk
				.getAoaPunkte("Abschnitt.090062siuf");

		assertEquals(liste.size(), 6);

		liste = bk.getAoaPunkte("Abschnitt.090062siuf", 1200, 1265);

		assertEquals(liste.size(), 1);

		BetriebsKilometerPunkt bkp = liste.get(0);

		assertEquals(bkp.getKilometrierung(), 657.0);
		assertEquals(bkp.getBlocknummer(), 0);
	}

	/**
	 * Methode fr
	 * {@link de.bsvrz.kex.bwsin.konfig.BetriebsKilometer#initialisiere(de.bsvrz.kex.bwsin.konfig.okstra.Okstra)}
	 * testen
	 */
	@Test
	public void testInitialisiere() {
		initOkstra();
	}
}
