/**
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information:<br>
 * BitCtrl Systems GmbH<br>
 * Wei&szlig;enfelser Stra&szlig;e 67<br>
 * 04229 Leipzig<br>
 * Phone: +49 341-490670<br>
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.NoSuchElementException;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.CommunicationError;
import de.bsvrz.dav.daf.main.ConnectionException;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.InconsistentLoginException;
import de.bsvrz.dav.daf.main.MissingParameterException;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.kex.bwsin.konfig.aktuell.SINAktuelleKonfiguration;
import de.bsvrz.kex.bwsin.konfig.aktuell.SINAktuellesObjekt;
import de.bsvrz.kex.bwsin.konfig.kb.SINKonfigurationsBereich;
import de.bsvrz.kex.bwsin.konfig.okstra.Okstra;
import de.bsvrz.sys.funclib.bitctrl.geolib.WGS84Polygon;
import de.bsvrz.sys.funclib.bitctrl.geolib.WGS84Punkt;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: TestKonfigTest.java 10461 2008-07-10 17:28:19Z gieseler $
 * 
 */
public class TestKonfigTest {

	/**
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
	}

	/**
	 * @throws java.lang.Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception {
	}

	private Okstra odb;
	private ClientDavConnection dav;
	private SINKonfigurationsBereich kb;
	private BufferedOutputStream _bos;
	private int level;
	private final String _style = "yellowLineGreenPoly";
	File kmlfile;

	/**
	 * @param ak
	 * @param kmlfile
	 */
	private void erzeugeKML(SINAktuelleKonfiguration ak, File kmlfile) {
		KML kml = new KML(kmlfile);

		kml.start("SINTEST", "aktuelle STS");

		// einlesen aller Objekte des Typs
		List<SINAktuellesObjekt> elemente_o = ak
				.getObjektListe("typ.straenTeilSegment");

		List<SINAktuellesObjekt> elemente = new ArrayList<SINAktuellesObjekt>(
				elemente_o);

		Collections.sort(elemente, new Comparator<SINAktuellesObjekt>() {
			public int compare(SINAktuellesObjekt o1, SINAktuellesObjekt o2) {
				return o1.getDavObjekt().getPid().compareTo(
						o2.getDavObjekt().getPid());
			}
		});

		for (SINAktuellesObjekt ao : elemente) {
			SystemObject so = ao.getDavObjekt();
			WGS84Polygon polygon = new WGS84Polygon(getPunktliste(ak,
					(ConfigurationObject) so));
			printData(kml, so, polygon);
		}

		kml.ende();
	}

	private ArrayList<WGS84Punkt> getPunktliste(SINAktuelleKonfiguration ak,
			ConfigurationObject co) {
		ArrayList<WGS84Punkt> punkte = new ArrayList<WGS84Punkt>();

		AttributeGroup ratg = ak.getAttributeGroup("atg.linienKoordinaten"); //$NON-NLS-1$
		Data rd = co.getConfigurationData(ratg);
		if (rd != null) {
			Data.NumberArray x = rd.getScaledArray("x"); //$NON-NLS-1$
			Data.NumberArray y = rd.getScaledArray("y"); //$NON-NLS-1$
			int anzahl = x.getLength();

			for (int i = 0; i < anzahl; i++) {
				WGS84Punkt p = new WGS84Punkt(x.getValue(i).doubleValue(), y
						.getValue(i).doubleValue());
				punkte.add(p);
			}
		}

		return punkte;
	}

	public void initDav() {
		String[] commandArguments = {
				// "-datenverteiler=127.0.0.1:8083",
				"-datenverteiler=10.44.44.10:8083",
				"-benutzer=Tester",
				"-authentifizierung=/home/gysi/workspace/DAV-Neu/Projekte/SIN-Test/skripte-bash/passwd",
				"-debugSetLoggerAndLevel=INFO",
				"-debugSetLoggerAndLevel=de.bsvrz.kex.bwsin:INFO",
				"-debugLevelStdErrText=INFO" };
		ArgumentList argumentList = new ArgumentList(commandArguments);
		try {
			String myname = "SIN-TEST";
			Debug.init(myname, argumentList);
			ClientDavParameters parameters = new ClientDavParameters(
					commandArguments);
			parameters.setApplicationName(myname);
			dav = new ClientDavConnection(parameters);
			dav.connect();
			dav.login();
		} catch (NoSuchElementException e) {
			// TODO Automatisch erstellter Catch-Block
			e.printStackTrace();
		} catch (MissingParameterException e) {
			// TODO Automatisch erstellter Catch-Block
			e.printStackTrace();
		} catch (CommunicationError e) {
			// TODO Automatisch erstellter Catch-Block
			e.printStackTrace();
		} catch (ConnectionException e) {
			// TODO Automatisch erstellter Catch-Block
			e.printStackTrace();
		} catch (InconsistentLoginException e) {
			// TODO Automatisch erstellter Catch-Block
			e.printStackTrace();
		}

	}

	private void print_filefooter(StringBuffer sb) {
		sb.append("\t</Document>\n");
		sb.append("</kml>\n");
		writeConfigData(sb);
	}

	/**
	 * <?xml version="1.0" encoding="UTF-8"?> <kml
	 * xmlns="http://earth.google.com/kml/2.1"> <Document> <name>Paths</name>
	 * <description>Examples of paths. Note that the tessellate tag is by
	 * default set to 0. If you want to create tessellated lines, they must be
	 * authored (or edited) directly in KML.</description> <Style
	 * id="yellowLineGreenPoly"> <LineStyle> <color>7f00ffff</color> <width>4</width>
	 * </LineStyle> <PolyStyle> <color>7f00ff00</color> </PolyStyle> </Style>
	 * <Placemark> <name>Absolute Extruded</name> <description>Transparent
	 * green wall with yellow outlines</description>
	 * <styleUrl>#yellowLineGreenPoly</styleUrl> <LineString> <extrude>1</extrude>
	 * <tessellate>1</tessellate> <altitudeMode>absolute</altitudeMode>
	 * <coordinates> -112.2550785337791,36.07954952145647,2357
	 * -112.2549277039738,36.08117083492122,2357
	 * -112.2552505069063,36.08260761307279,2357
	 * -112.2564540158376,36.08395660588506,2357
	 * -112.2580238976449,36.08511401044813,2357
	 * -112.2595218489022,36.08584355239394,2357
	 * -112.2608216347552,36.08612634548589,2357
	 * -112.262073428656,36.08626019085147,2357
	 * -112.2633204928495,36.08621519860091,2357
	 * -112.2644963846444,36.08627897945274,2357
	 * -112.2656969554589,36.08649599090644,2357 </coordinates> </LineString>
	 * </Placemark> </Document> </kml>
	 */

	private void print_fileheader(StringBuffer sb) {
		sb.append("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n");
		sb.append("<kml xmlns=\"http://earth.google.com/kml/2.1\">\n");
		sb.append("\t<Document>\n");
		sb.append("\t\t<name>STS-SIN</name>\n");
		sb.append("\t\t<description>STS generiert aus SIN</description>\n");

		sb.append("\t\t\t<Style id=\"" + _style + "\">\n");
		sb.append("\t\t\t\t<LineStyle>\n");
		sb.append("\t\t\t\t\t<color>7f00ffff</color>\n");
		sb.append("\t\t\t\t\t<width>4</width>\n");
		sb.append("\t\t\t</LineStyle>\n");
		sb.append("\t\t\t\t<PolyStyle>\n");
		sb.append("\t\t\t\t\t<color>7f00ff00</color>\n");
		sb.append("\t\t\t\t</PolyStyle>\n");
		sb.append("\t\t\t</Style>\n");

		writeConfigData(sb);

		level = 3;
	}

	private void printData(KML kml, SystemObject so, WGS84Polygon polygon) {
		/**
		 * <Placemark> <name>Absolute Extruded</name> <description>Transparent
		 * green wall with yellow outlines</description>
		 * <styleUrl>#yellowLineGreenPoly</styleUrl> <LineString> <extrude>1</extrude>
		 * <tessellate>1</tessellate> <altitudeMode>absolute</altitudeMode>
		 * <coordinates> -112.2550785337791,36.07954952145647,2357
		 * -112.2549277039738,36.08117083492122,2357
		 * -112.2552505069063,36.08260761307279,2357
		 * -112.2564540158376,36.08395660588506,2357
		 * -112.2580238976449,36.08511401044813,2357
		 * -112.2595218489022,36.08584355239394,2357
		 * -112.2608216347552,36.08612634548589,2357
		 * -112.262073428656,36.08626019085147,2357
		 * -112.2633204928495,36.08621519860091,2357
		 * -112.2644963846444,36.08627897945274,2357
		 * -112.2656969554589,36.08649599090644,2357 </coordinates>
		 * </LineString> </Placemark>
		 */
		System.out.println("Objekt: " + so.getPid() + " wird geschrieben");

		kml.appendLine("<Placemark>");
		kml.inclevel();
		kml.appendLine("<name>" + so.getPid() + "</name>");
		kml.appendLine("<description>" + so.getName() + "</description>");
		kml.appendLine("<styleUrl>#" + _style + "</styleUrl>");
		kml.appendLine("<LineString>");
		kml.inclevel();
		kml.appendLine("<extrude>1</extrude>");
		kml.appendLine("<tessellate>1</tessellate>");
		kml.appendLine("<altitudeMode>absolute</altitudeMode>");
		kml.appendLine("<coordinates>");
		DecimalFormat df = new DecimalFormat(".#############");
		DecimalFormatSymbols dfs = new DecimalFormatSymbols();
		dfs.setDecimalSeparator('.');
		df.setDecimalFormatSymbols(dfs);
		for (WGS84Punkt punkt : polygon.getKoordinaten()) {
			kml.appendLine(df.format(punkt.getLaenge()) + ","
					+ df.format(punkt.getBreite()));
		}

		kml.appendLine("</coordinates>");
		kml.declevel();
		kml.appendLine("</LineString>");
		kml.declevel();
		kml.appendLine("</Placemark>");
	}

	/**
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception {
	}

	/**
	 * @throws java.lang.Exception
	 */
	@After
	public void tearDown() throws Exception {
	}

	/**
	 * Erzeugt ein Googlo-Earth KML-File mit den STS
	 */
	@Test
	public void testKML() {
		initDav();

		SINAktuelleKonfiguration ak = new SINAktuelleKonfiguration();
		ak.initialisiere(dav, "kb.sin");

		kmlfile = new File(
				"/home/gysi/workspace/DAV-Neu/Projekte/SIN-Test/konfiguration/kv.bw.bitctrl/xml/kb.sin_dav.kml");

		erzeugeKML(ak, kmlfile);
	}

	// @Test
	public void testList() {
		InputStream is = ClassLoader
				.getSystemResourceAsStream("de/bsvrz/kex/bwsin/vew/wfs_bw_Default.properties");

		List<String> _neue_objekte = new ArrayList<String>();
		List<String> aktuelle_objekte = new ArrayList<String>();
		List<String> geloeschte_objekte;

		List<String> neue = new ArrayList<String>();
		List<String> alte = new ArrayList<String>();
		List<String> shared = new ArrayList<String>();

		_neue_objekte.add("a");
		_neue_objekte.add("b");
		_neue_objekte.add("c");
		_neue_objekte.add("d");

		aktuelle_objekte.add("b");
		aktuelle_objekte.add("c");
		aktuelle_objekte.add("e");

		List<String> ueberpruefen = new ArrayList<String>(_neue_objekte);
		ueberpruefen.retainAll(aktuelle_objekte);

		List<String> dup = new ArrayList<String>(_neue_objekte);
		_neue_objekte.removeAll(aktuelle_objekte);

		aktuelle_objekte.removeAll(dup);

		geloeschte_objekte = aktuelle_objekte;
	}

	/**
	 * Schreibt einen Stringbuffer in Datei.
	 * 
	 * @param sb
	 *            Zu schreibender StringBuffer.
	 */
	private void writeConfigData(StringBuffer sb) {
		try {
			_bos.write(sb.toString().getBytes());
		} catch (IOException e) {
			System.out.println("Fehler beim Schreiben auf Datei: "
					+ kmlfile.getAbsolutePath());
			e.printStackTrace();
			throw new RuntimeException(e);
		}
	}

}
