/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig.aktuell;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.bitctrl.geolib.WGS84Koordinate;

/**
 * Aufteilungspunkt aus der aktuellen Konfiguration.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: AufteilungspunktAktuelleKonfiguration.java 10390 2008-07-09 15:06:39Z gieseler $
 * 
 */

public class AufteilungspunktAktuelleKonfiguration {

	/**
	 * Die Koordinaten.
	 */
	private WGS84Koordinate koordinaten = null;

	/**
	 * Die Linienreferenz.
	 */
	private String linienReferenz = null;

	/**
	 * Der Offset auf der Linie.
	 */
	private float linienReferenzOffset;

	/**
	 * Das zugeh&ouml;rige Systemobjekt.
	 */
	SystemObject systemObjekt;

	/**
	 * Konstruktor.
	 * 
	 * @param objekt
	 *            Systemobjekt
	 * @param dav
	 *            DaV-Verbindung
	 */
	public AufteilungspunktAktuelleKonfiguration(SystemObject objekt,
			final ClientDavConnection dav) {

		systemObjekt = objekt;

		// Punktkoordinaten einlesen
		SystemObjectType dt = dav.getDataModel().getType("typ.punktXY");
		if (dt.equals(objekt.getType()) || objekt.getType().inheritsFrom(dt)) {
			AttributeGroup ratg = dav.getDataModel().getAttributeGroup(
					"atg.punktKoordinaten"); //$NON-NLS-1$
			Data rd = objekt.getConfigurationData(ratg);
			if (rd != null) {
				koordinaten = new WGS84Koordinate(rd.getScaledValue("x")
						.doubleValue(), rd.getScaledValue("y").doubleValue());
			}
		}

		// Linienreferenz einlesen
		dt = dav.getDataModel().getType("typ.punktLiegtAufLinienObjekt");
		// TODO inheritsFrom funktioniert nicht!!!
		// if (dt.equals(objekt.getType()) || objekt.getType().inheritsFrom(dt))
		// {
		AttributeGroup ratg = dav.getDataModel().getAttributeGroup(
				"atg.punktLiegtAufLinienObjekt"); //$NON-NLS-1$
		Data rd = objekt.getConfigurationData(ratg);
		if (rd != null) {
			linienReferenz = rd.getReferenceValue("LinienReferenz")
					.getValueText();
			linienReferenzOffset = rd.getScaledValue("Offset").floatValue();
		}
		// }
	}

	/**
	 * Gibt die Koordinaten zur&uuml;ck.
	 * 
	 * @return Koordinaten
	 */
	public WGS84Koordinate getKoordinaten() {
		return koordinaten;
	}

	/**
	 * Gibt die Linienreferenz zur&uuml;ck.
	 * 
	 * @return the linienReferenz
	 */
	public String getLinienReferenz() {
		return linienReferenz;
	}

	/**
	 * Gibt den Offset auf dem Linienelement zur&uuml;ck.
	 * 
	 * @return the linienReferenzOffset
	 */
	public float getLinienReferenzOffset() {
		return linienReferenzOffset;
	}

	/**
	 * Gibt das Systemobjekt zur&uuml;ck.
	 * 
	 * @return Systemobjekt
	 */
	public SystemObject getSystemObjekt() {
		return systemObjekt;
	}

	/**
	 * Test, ob das Objekt Koordinaten besitzt.
	 * 
	 * @return true, wenn das Objekt Koordinaten besitzt, sonst false
	 */
	public boolean hatKoordinaten() {
		return koordinaten != null;
	}

	/**
	 * Test, ob das Objekt eine Linienreferenz besitzt.
	 * 
	 * @return true, wenn das Objekt eine Linienreferenz besitzt, sonst false
	 */
	public boolean hatLinienreferenz() {
		return linienReferenz != null;
	}

}
