/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig.aktuell;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ObjectTimeSpecification;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Modul Aktuelle Konfiguration.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: SINAktuelleKonfiguration.java 10390 2008-07-09 15:06:39Z gieseler $
 * 
 */
public class SINAktuelleKonfiguration implements
		SINAktuelleKonfigurationInterface {

	/**
	 * Liste aller relevanten Objekttypen des TeilModellVerkehr.
	 */
	public static final String[] OBJEKTTYPEN_TEILMODELL_VERKEHR = new String[] {
			"typ.straenSegment", "typ.strae", "typ.straenTeilSegment",
			"typ.straenKnoten" };
	/**
	 * DebugLogger fr Debug-Ausgaben.
	 */
	private final Debug debug = Debug.getLogger();

	/**
	 * Datenmodell.
	 */
	private ConfigurationArea kbConfig = null;

	/**
	 * Typ-Cache.
	 */
	private final Map<String, List<SINAktuellesObjekt>> typeCache = new HashMap<String, List<SINAktuellesObjekt>>();

	/**
	 * Objekt-Cache.
	 */
	private final Map<String, SINAktuellesObjekt> objektCache = new HashMap<String, SINAktuellesObjekt>();

	/**
	 * Datenverteiler-Verbindung.
	 */
	private ClientDavConnection dav;

	/**
	 * Liste der nicht verabeiteten koordinantenbezogenen Aufteilungspunkte.
	 */
	private List<AufteilungspunktAktuelleKonfiguration> nichtVerarbeiteteAufteilungspunkteKoordinaten = null;

	/**
	 * Liste der nicht verabeiteten Aufteilungspunkte mit Linienreferenzen.
	 */
	private List<AufteilungspunktAktuelleKonfiguration> nichtVerarbeiteteAufteilungspunkteLinienreferenz = null;

	/**
	 * Gibt eine Liste der PIDs aller relevanten Objekte des
	 * TeilmodellesVerkehrGlobal zur&uuml;ck.
	 * 
	 * @return Liste mit PIDs
	 */
	public List<String> getAktuelleObjekteTeilmodellVerkehr() {
		List<String> ret = new ArrayList<String>();

		if (kbConfig != null) {
			for (int i = 0; i < OBJEKTTYPEN_TEILMODELL_VERKEHR.length; i++) {
				for (SINAktuellesObjekt so : getObjektListe(OBJEKTTYPEN_TEILMODELL_VERKEHR[i])) {
					ret.add(so.getPid());
				}
			}
		}

		return ret;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.aktuell.SINAktuelleKonfigurationInterface#getAQ()
	 */
	public List<SystemObject> getAQ() {
		return dav.getDataModel().getType("typ.anzeigeQuerschnitt")
				.getElements();
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.aktuell.SINAktuelleKonfigurationInterface#getAttributeGroup(java.lang.String)
	 */
	public AttributeGroup getAttributeGroup(String atg) {
		return dav.getDataModel().getAttributeGroup(atg);
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.aktuell.SINAktuelleKonfigurationInterface#getFreieMarker()
	 */
	public List<SystemObject> getFreieMarker() {
		return dav.getDataModel().getType("typ.freierMarkerStraenSegment")
				.getElements();
	}

	/**
	 * Gibt die Liste der nicht verabeiteten koordinantenbezogenen
	 * Aufteilungspunkte zur&uuml;ck..
	 * 
	 * @return Aufteilungspunkte
	 */
	public List<AufteilungspunktAktuelleKonfiguration> getNichtVerarbeiteteAufteilungspunkteKoordinaten() {
		return nichtVerarbeiteteAufteilungspunkteKoordinaten;
	}

	/**
	 * Gibt die Liste der nicht verabeiteten Aufteilungspunkte mit
	 * Linienreferenzen zur&uuml;ck.
	 * 
	 * @return Aufteilungspunkte
	 */
	public List<AufteilungspunktAktuelleKonfiguration> getNichtVerarbeiteteAufteilungspunkteLinienreferenz() {
		return nichtVerarbeiteteAufteilungspunkteLinienreferenz;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.aktuell.SINAktuelleKonfigurationInterface#getObjekt(java.lang.String)
	 */
	public SINAktuellesObjekt getObjekt(final String pid) {
		SINAktuellesObjekt objekt = objektCache.get(pid);

		if (objekt == null && kbConfig != null) {
			ConfigurationObject davobjekt = (ConfigurationObject) dav
					.getDataModel().getObject(pid);
			if (davobjekt != null) {
				objekt = new SINAktuellesObjekt(davobjekt);
				objektCache.put(objekt.getPid(), objekt);
			}
		}

		return objekt;
	}

	/**
	 * Gibt ein Objekt aus dem Cache zur&uuml;ck bzw. legt ein neues an.
	 * 
	 * @param systemObjekt
	 *            Systemobjekt
	 * @return Objekt
	 */
	private SINAktuellesObjekt getObjekt(final SystemObject systemObjekt) {
		SINAktuellesObjekt objekt = objektCache.get(systemObjekt.getPid());

		if (objekt == null) {
			objekt = new SINAktuellesObjekt((ConfigurationObject) systemObjekt);
			objektCache.put(objekt.getPid(), objekt);
		}

		return objekt;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.aktuell.SINAktuelleKonfigurationInterface#getObjektListe(java.lang.String)
	 */
	public List<SINAktuellesObjekt> getObjektListe(final String davtyp) {
		List<SINAktuellesObjekt> liste = null;

		if (kbConfig != null) {
			liste = typeCache.get(davtyp);
			if (liste == null) {
				liste = new ArrayList<SINAktuellesObjekt>();

				Collection<SystemObject> objekte = dav.getDataModel()
						.getObjects(
								Collections.singleton(kbConfig),
								Collections.singleton(dav.getDataModel()
										.getType(davtyp)),
								ObjectTimeSpecification.valid());

				for (SystemObject so : objekte) {
					liste.add(getObjekt(so));
				}

				typeCache.put(davtyp, liste);
			}
		} else {
			// gibt leere Liste zurck
			liste = new ArrayList<SINAktuellesObjekt>();
		}

		return liste;
	}

	/**
	 * Gibt alle Objekte eines Typ des gesamten Datenkataloges zur&uuml;ck.
	 * 
	 * @param davtyp
	 *            Typename im Datenkatalog
	 * @return Objektliste
	 */
	public List<SINAktuellesObjekt> getObjektListeGlobal(final String davtyp) {
		List<SINAktuellesObjekt> liste = new ArrayList<SINAktuellesObjekt>();

		Collection<SystemObject> objekte = dav.getDataModel().getType(davtyp)
				.getElements();

		for (SystemObject so : objekte) {
			liste.add(getObjekt(so));
		}

		return liste;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.aktuell.SINAktuelleKonfigurationInterface#initialisiere(de.bsvrz.dav.daf.main.ClientDavConnection,
	 *      java.lang.String)
	 */
	public void initialisiere(final ClientDavConnection davConnection,
			final String kbpid) {
		this.dav = davConnection;
		this.kbConfig = davConnection.getDataModel()
				.getConfigurationArea(kbpid);

		if (kbConfig == null) {
			debug
					.warning("Es existiert keine aktuelle Konfiguration fr den Konfigurationsbereich '"
							+ kbpid + "'");
		}
	}

	/**
	 * Setzt die Liste der nicht verabeiteten koordinantenbezogenen
	 * Aufteilungspunkte.
	 * 
	 * @param punkte
	 *            Aufteilungspunkte
	 */
	public void setNichtVerarbeiteteAufteilungspunkteKoordinaten(
			final List<AufteilungspunktAktuelleKonfiguration> punkte) {
		nichtVerarbeiteteAufteilungspunkteKoordinaten = punkte;
	}

	/**
	 * Setzt die Liste der nicht verabeiteten Aufteilungspunkte mit
	 * Linienreferenz.
	 * 
	 * @param punkte
	 *            Aufteilungspunkte
	 */
	public void setNichtVerarbeiteteAufteilungspunkteLinienreferenz(
			final List<AufteilungspunktAktuelleKonfiguration> punkte) {
		nichtVerarbeiteteAufteilungspunkteLinienreferenz = punkte;
	}
}
