/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig.aktuell;

import java.util.List;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Interface zur aktuellen Konfiguration.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: SINAktuelleKonfigurationInterface.java 7934 2008-04-03 17:41:24Z gieseler $
 * 
 */
public interface SINAktuelleKonfigurationInterface {

	/**
	 * Diese Methode gibt eine Liste aller AQ der aktuellen Konfiguration
	 * zur&uuml;ck.
	 * 
	 * @return Feld aller AQ der aktuellen Konfiguration
	 */
	List<SystemObject> getAQ();

	/**
	 * Diese Methode gibt die Liste aller Objekte eines Typs der aktuellen
	 * Konfiguration zur&uuml;ck.
	 * 
	 * @param atg
	 *            Name der Attributgruppe.
	 * @return Liste mit Objekten der aktuellen Konfiguration
	 */
	AttributeGroup getAttributeGroup(String atg);

	/**
	 * Diese Methode gibt eine Liste aller "Freien Marker" der aktuellen
	 * Konfiguration zur&uuml;ck.
	 * 
	 * @return Feld aller "Freien Marker" der aktuellen Konfiguration
	 */
	List<SystemObject> getFreieMarker();

	/**
	 * Dieser Methode gibt ein Konfigurationsobjekt mit der PID pid zur&uuml;ck,
	 * falls es existiert.
	 * 
	 * @param pid
	 *            die angefragte PID
	 * @return ConfigurationObject
	 */
	SINAktuellesObjekt getObjekt(String pid);

	/**
	 * Diese Methode gibt die Liste aller Objekte eines Typs der aktuellen
	 * Konfiguration zur&uuml;ck.
	 * 
	 * @param davtyp
	 *            Name des Typs der angeforderten Objekte.
	 * @return Liste mit Objekten der aktuellen Konfiguration
	 */
	List<SINAktuellesObjekt> getObjektListe(String davtyp);

	/**
	 * Die Methode veranlasst das Modul, die aktuelle Konfiguration des
	 * Datenverteilers einzulesen.
	 * 
	 * @param davConnection
	 *            {@link ClientDavConnection}
	 * @param kbpid
	 *            die PID des Konfigurationsbereiches
	 */
	void initialisiere(ClientDavConnection davConnection, String kbpid);
}
