/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig.kb;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.kex.bwsin.konfig.Route;
import de.bsvrz.kex.bwsin.konfig.SegmentFactory;
import de.bsvrz.kex.bwsin.konfig.StrassenTeilAbschnitt;
import de.bsvrz.sys.funclib.bitctrl.geolib.WGS84Polygon;
import de.bsvrz.sys.funclib.bitctrl.geolib.WGS84Punkt;

/**
 * Dummy-Inneres Stra&szlig;ensegment zur Verbindung von 2 unmittelbar
 * aufeinander folgenden &auml;u&szlig;eren Stra&szlig;ensegmenten.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: DummyISS.java 10083 2008-07-01 18:34:53Z gieseler $
 * 
 */

public class DummyISS extends SINISS {

	/**
	 * Das Teilsegment, aus dem das Segment besteht.
	 */
	private KonfigurationsObjekt teilsement = null;

	/**
	 * Erzeugt ein Dummy-ISS.
	 * 
	 * @param iss
	 *            Inneres Stra&szlig;ensegment
	 * @param route
	 *            Route
	 * @param segmentFactory
	 *            Segment-Factory
	 */
	public DummyISS(SINISS iss, Route route, SegmentFactory segmentFactory) {
		super(iss.getPid(), iss.getName());
		// letzte Koordinate letztes STS
		List<StrassenTeilAbschnitt> ta = route.getFirst().getTeilabschnitte();
		StrassenTeilAbschnitt sta = ta.get(ta.size() - 1);

		WGS84Punkt p = sta.getKoordinaten()
				.get(sta.getKoordinaten().size() - 1);
		ArrayList<WGS84Punkt> k = new ArrayList<WGS84Punkt>();
		// 2 mal gleicher Punkt, damit Linienrelationen
		// "von/nach" funktionieren
		k.add(p);
		k.add(p);

		String stspid = segmentFactory
				.bildePIDStrassenTeilSegment(new WGS84Polygon(k));
		String stsname = SegmentFactory.bildeSegmentNamen(stspid, "DummySTS"); //$NON-NLS-1$
		SINSTS sts = new SINSTS(stspid, stsname);

		sts.setKoordinaten(k);
		sts.setLaenge(0);
		sts.setRichtung(route.getFirst().getRichtung());

		// erbe Eigenschaften des letzten vorangegangenen Teilabschnittes
		sts.eigenschaftenFeatures = sta.getEigenschaften();

		teilsement = sts;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.kb.AbstraktStrassenSegment#getTeilSegmente()
	 */
	@Override
	public List<KonfigurationsObjekt> getTeilSegmente() {
		List<KonfigurationsObjekt> teilsegmente = new ArrayList<KonfigurationsObjekt>();
		teilsegmente.add(teilsement);
		return teilsegmente;
	}
}
