/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig.kb;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.bwsin.konfig.ImportException;
import de.bsvrz.kex.bwsin.konfig.Location;
import de.bsvrz.kex.bwsin.vew.SINProperties;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.kbgen.ProxyConfigurationObject;
import de.bsvrz.sys.funclib.kbgen.ProxyDataDescription;

/**
 * SWE-interne Repr&auml;sentation eines &auml;u&szlig;eren
 * Stra&szligensegmentes.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: SINASS.java 9193 2008-05-26 12:57:36Z gieseler $
 * 
 */
public class SINASS extends AbstraktStrassenSegment {

	/**
	 * Logger f&uuml;r Ausgaben.
	 */
	private static final Debug DEBUG = Debug.getLogger();

	/**
	 * Stra&szlig;enknoten, von dem sich das Stra&szlig;ensegment wegbewegt.
	 */
	private KonfigurationsObjekt vonKnoten = null;

	/**
	 * Stra&szlig;enknoten, auf den sich das Stra&szlig;ensegment zubewegt.
	 */
	private KonfigurationsObjekt nachKnoten = null;

	/**
	 * Konstruktor mit PID und Name.
	 * 
	 * @param pid
	 *            PID
	 * @param name
	 *            Name
	 */
	public SINASS(String pid, String name) {
		super(pid, name);
	}

	/**
	 * Gibt den {@link SINStrassenKnoten} zur&uuml;ck, auf den sich das
	 * Stra&szlig;ensegment zubewegt.
	 * 
	 * @return {@link KonfigurationsObjekt}
	 */
	public KonfigurationsObjekt getNachKnoten() {
		return nachKnoten;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.kb.KonfigurationsObjekt#getProxyObjekt(de.bsvrz.dav.daf.main.ClientDavConnection)
	 */
	@SuppressWarnings("nls")
	public ProxyConfigurationObject getProxyObjekt(ClientDavConnection dav)
			throws ImportException {
		ProxyConfigurationObject pco = new ProxyConfigurationObject(pid, name,
				dav.getDataModel().getType("typ.ueresStraenSegment"));

		davtyp = pco.getType();

		// typ.linie
		Data data = createData(dav, "atg.linie", "asp.eigenschaften");
		data.getScaledValue("Lnge").set(getLaenge());
		pco.addConfigurationData(new ProxyDataDescription(data, null));

		// typ.bestehtAusLinienObjekten
		// atg.bestehtAusLinienObjekten
		data = createData(dav, "atg.bestehtAusLinienObjekten",
				"asp.eigenschaften");
		data.setToDefault();
		Data.ReferenceArray lr = data.getReferenceArray("LinienReferenz");
		List<KonfigurationsObjekt> teilsegmente = getTeilSegmente();
		lr.setLength(teilsegmente.size());
		List<String> linienrefs = new ArrayList<String>(teilsegmente.size());
		for (int i = 0; i < teilsegmente.size(); i++) {
			linienrefs.add(teilsegmente.get(i).getPid());
		}
		pco.addConfigurationData(new ProxyDataDescription(data, linienrefs));

		// typ.straenSegment
		// atg.straenSegment
		data = createData(dav, "atg.straenSegment", "asp.eigenschaften");
		data.setToDefault();
		if (zuStrasse == null) {
			DEBUG
					.error("ASS: "
							+ pid
							+ " 'gehrtZuStrae' kann nicht gebildet werden, da kein Strassenobjekt");
		}
		// Referenz "gehrtZuStrae"
		List<String> strasserefs = new ArrayList<String>();
		strasserefs.add(zuStrasse == null ? "" : zuStrasse.getPid());
		data.getScaledValue("Lnge").set(getLaenge());
		pco.addConfigurationData(new ProxyDataDescription(data, strasserefs));

		// typ.ueresStraenSegment
		// atg.ueresStraenSegment
		data = createData(dav, "atg.ueresStraenSegment", "asp.eigenschaften");
		List<String> assrefs = new ArrayList<String>();
		// Referenz "vonKnoten"
		assrefs.add(vonKnoten == null ? "" : vonKnoten.getPid());
		// Referenz "nachKnoten"
		assrefs.add(nachKnoten == null ? "" : nachKnoten.getPid());
		// Referenz "TmcPrimrOrt"
		if (SINProperties.getInstance().istAktiv(
				SINProperties.PROP_KEY_TMCREFERENZEN)) {
			assrefs.add("tmclc." + locationNach.getLocationCode());
		} else {
			assrefs.add(null);
		}

		if (locationNach.getLocationRichtung() == Location.LOCATION_RICHTUNG_POSITIV) {
			data.getUnscaledValue("TmcRichtung").set(1);
		}
		if (locationNach.getLocationRichtung() == Location.LOCATION_RICHTUNG_NEGATIV) {
			data.getUnscaledValue("TmcRichtung").set(-1);
		}

		pco.addConfigurationData(new ProxyDataDescription(data, assrefs));

		return pco;
	}

	/**
	 * Gibt den {@link SINStrassenKnoten} zur&uuml;ck, von dem sich das
	 * Stra&szlig;ensegment wegbewegt.
	 * 
	 * @return {@link KonfigurationsObjekt}
	 */
	public KonfigurationsObjekt getVonKnoten() {
		return vonKnoten;
	}

	/**
	 * Setzt den Stra&szlig;enknoten, auf den sich das Stra&szlig;ensegment
	 * zubewegt.
	 * 
	 * @param knoten
	 *            {@link KonfigurationsObjekt}
	 */
	public void setNachKnoten(KonfigurationsObjekt knoten) {
		nachKnoten = knoten;
	}

	/**
	 * Setzt den {@link SINStrassenKnoten}, von dem sich das
	 * Stra&szlig;ensegment wegbewegt.
	 * 
	 * @param knoten
	 *            {@link KonfigurationsObjekt}
	 */
	public void setVonKnoten(KonfigurationsObjekt knoten) {
		vonKnoten = knoten;
	}

}
