/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig.kb;

import java.util.Scanner;
import java.util.regex.Pattern;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.bwsin.konfig.ImportException;
import de.bsvrz.kex.bwsin.konfig.lcl.LCL;
import de.bsvrz.sys.funclib.kbgen.ProxyConfigurationObject;
import de.bsvrz.sys.funclib.kbgen.ProxyDataDescription;

/**
 * SWE-interne Repr&auml;sentation einer Stra&szlig;e.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: SINStrasse.java 10390 2008-07-09 15:06:39Z gieseler $
 * 
 */
public class SINStrasse extends AbstraktKonfigurationsObjekt {

	/**
	 * Erzeugt einen Namen f&uuml;r eine Stra&szlig;e. Der Name wird
	 * entsprechend Afo gebildet.
	 * 
	 * @param locationcode
	 *            Locationcode der Stra&szlig;e
	 * @return Name
	 */
	public static String bildeNamen(String locationcode) {
		LCL lcl = LCL.getInstance();
		String bezeichnung = lcl.getLocationCodeWert(locationcode,
				LCL.ROADNUMBER);
		String anschlussvon = lcl.getLocationCodeWert(locationcode,
				LCL.FIRST_NAME);
		String anschlussbis = lcl.getLocationCodeWert(locationcode,
				LCL.SECOND_NAME);
		String zusatzinfo = lcl.getLocationCodeWert(locationcode, LCL.ROADNAME);

		String name = bezeichnung + " von " + anschlussvon + " nach " + //$NON-NLS-1$ //$NON-NLS-2$
				anschlussbis;

		if (!(zusatzinfo.length() == 0)) {
			name += "(" + zusatzinfo + ")"; //$NON-NLS-1$ //$NON-NLS-2$
		}

		return name;
	}

	/**
	 * Erzeugt eine PID f&uuml;r eine Stra&szlig;e. Die PID wird entsprechend
	 * Afo gebildet.
	 * 
	 * @param typ
	 *            Stra&szlig;entyp als String
	 * @param nummer
	 *            Stra&szlig;entyp als String
	 * @param zusatz
	 *            Zusatz in der PID
	 * @param locationcode
	 *            Locationcode der Stra&szlig;e
	 * 
	 * @return PID
	 */
	private static String bildePID(String typ, String nummer, String zusatz,
			String locationcode) {
		String pid = "strasse." + typ + "." + nummer; //$NON-NLS-1$//$NON-NLS-2$
		if (zusatz != null && !(zusatz.length() == 0)) {
			pid += "." + zusatz; //$NON-NLS-1$
		}

		pid += ".tmclc." + String.format("%05d", Integer.parseInt(locationcode)); //$NON-NLS-1$ //$NON-NLS-2$

		return pid;
	}

	/**
	 * Wandelt einen Stra&szlig;entyp aus der LCL in den Typ entsprechend DatK
	 * um. DatK: SonstigeStra&szlig;e 0 Sonstiger Stra&szlig;entyp. Autobahn 1
	 * Strae ist Autobahn. Bundesstrae 2 Strae ist Bundesstra&szlig;e.
	 * Landstrae 3 Strae ist Landstra&szlig;e. Kreisstrae 4 Strae ist
	 * Kreisstra&szlig;e. Stadtstrae 5 Strae ist Stadtstra&szlig;e.
	 * Hauptverkehrsstrae 6 Strae ist st&auml;dtische
	 * Hauptverkehrsstra&szlig;e. Sammelstrae 7 Strae ist st&auml;dtische
	 * Sammelstra&szlig;e. Anliegerstrae 8 Strae ist st&auml;dtische
	 * Anliegerstra&szlig;e.
	 * 
	 * Bei Eintr&auml;gen, die nicht zugeordnet werden k&ouml;nnen, wird der
	 * DatK Wert auf 'SonstigeStra&szlig;e' gesetzt.
	 * 
	 * @param lclstrassentyp
	 *            Stra&szlig;entyp aus LCL
	 * @return Dav-Stra&szlig;typ
	 */
	@SuppressWarnings("nls")
	private static int davStrasseVonLcl(String lclstrassentyp) {
		if (lclstrassentyp.equals("A")) {
			return 1;
		} else if (lclstrassentyp.equals("B")) {
			return 2;
		} else if (lclstrassentyp.equals("L")) {
			return 3;
		} else if (lclstrassentyp.equals("K")) {
			return 4;
		} else if (lclstrassentyp.equals("G")) {
			return 5;
		} else {
			return 0;
		}
	}

	/**
	 * Erzeugt ein neues Stra&szlig;en-Objekt auf der Basis eines Locationcodes.
	 * Die Stra&szlig;ennummer der LCL wird wie folgt in die PID konvertiert: -
	 * der erste Buchstabe wird zur Bildung des Stra&szlig;entyps benutzt, - die
	 * erste Folge von Ziffern wird als Stra&szlig;ennummer benutzt, - alle
	 * weiteren Zeichen der LCL-Bezeichnung (sowohl vor als auch nach der
	 * Nummer) werden in dem Feld 'zusatz' zusammengefa&szlig;t
	 * 
	 * Beispiel:
	 * 
	 * LCL-Bezeichnung: G KA01 PID: strasse.G.01.KA.tmclc.30878
	 * 
	 * @param locationcode
	 *            Locationcode der Stra&szlig;e
	 * @return {@link SINStrasse}
	 * @throws ImportException
	 *             bei Ausnahmen
	 */
	public static SINStrasse erzeugeStrasse(String locationcode)
			throws ImportException {
		LCL lcl = LCL.getInstance();
		String lclroadnr = lcl
				.getLocationCodeWert(locationcode, LCL.ROADNUMBER);

		String typ = lclroadnr.substring(0, 1);

		// bestimme Nummer
		Scanner s = new Scanner(lclroadnr.substring(1));
		Pattern p = Pattern.compile("[0-9]+"); //$NON-NLS-1$
		String nummer = s.findInLine(p);
		String[] zusaetze = lclroadnr.substring(1).split(nummer);

		String zusatz = ""; //$NON-NLS-1$
		for (int i = 0; i < zusaetze.length; i++) {
			zusatz += zusaetze[i].trim();
		}

		String pid = bildePID(typ, nummer, zusatz, locationcode);
		String name = bildeNamen(locationcode);

		SINStrasse strasse = new SINStrasse(pid, name);
		strasse.setTyp(davStrasseVonLcl(typ));
		if (nummer == null || nummer.length() == 0) {
			throw new ImportException("Die Nummer der Strasse " + lclroadnr + //$NON-NLS-1$
					"(LC: " + locationcode + ") kann nicht bestimmt werden!"); //$NON-NLS-1$//$NON-NLS-2$
		}
		try {
			strasse.setNummer(Integer.parseInt(nummer));
		} catch (NumberFormatException nfe) {
			throw new ImportException("Die Nummer der Strasse " + lclroadnr + //$NON-NLS-1$
					"(LC: " + locationcode + ") kann nicht bestimmt werden!"); //$NON-NLS-1$ //$NON-NLS-2$
		}

		strasse.setZusatz(zusatz);
		strasse.setLocationCode(locationcode);

		return strasse;
	}

	/**
	 * Typ der Stra&szlig;e.
	 */
	private int typ;

	/**
	 * Nummer der Stra&szlig;e.
	 */
	private int nummer;

	/**
	 * Zusatzname der Stra&szlig;e.
	 */
	private String zusatz;

	/**
	 * Locationcode der Stra&szlig;e.
	 */
	private String locationcode;

	/**
	 * Konstruktor mit PID und Name.
	 * 
	 * @param pid
	 *            PID
	 * @param name
	 *            Name
	 */
	public SINStrasse(String pid, String name) {
		super(pid, name);
	}

	/**
	 * Gibt den Locationcode der Stra&szlig;e zur&uuml;ck.
	 * 
	 * @return Locationcode
	 */
	public String getLocationCode() {
		return locationcode;
	}

	/**
	 * Gibt die Stra&szlig;ennummer der Stra&szlig;e zur&uuml;ck.
	 * 
	 * @return Stra&szlig;ennummer
	 */
	public int getNummer() {
		return nummer;
	}

	/**
	 * Versorgt alle Attribute und erzeugt ein {@link ProxyConfigurationObject}.
	 * 
	 * @param dav
	 *            Datenverteiler-Verbindung
	 * 
	 * @return {@link ProxyConfigurationObject}
	 */
	@SuppressWarnings("nls")
	public ProxyConfigurationObject getProxyObjekt(ClientDavConnection dav) {
		ProxyConfigurationObject pco = new ProxyConfigurationObject(pid, name,
				dav.getDataModel().getType("typ.strae"));

		davtyp = pco.getType();

		// atg.strae
		Data data = createData(dav, "atg.strae", "asp.eigenschaften");
		data.setToDefault();
		data.getUnscaledValue("Typ").set(typ);
		data.getUnscaledValue("Nummer").set(nummer);
		data.getTextValue("Zusatz").setText(zusatz);
		pco.addConfigurationData(new ProxyDataDescription(data, null));

		return pco;
	}

	/**
	 * Gibt den Typ der Stra&szlig;e zur&uuml;ck.
	 * 
	 * @return Stra&szlig;etyp entsprechend DatK
	 */
	public int getTyp() {
		return typ;
	}

	/**
	 * Gibt die Zusatzbezeichnung der Stra&szlig;e zur&uuml;ck.
	 * 
	 * @return Zusatzbezeichnung der Stra&szlig;e
	 */
	public String getZusatz() {
		return zusatz;
	}

	/**
	 * Setzt den Locationcode der Stra&szlig;e.
	 * 
	 * @param location
	 *            Locationcode
	 */
	public void setLocationCode(String location) {
		this.locationcode = location;
	}

	/**
	 * Setzt die Nummer der Stra&szlig;e.
	 * 
	 * @param nummer
	 *            Stra&szlig;ennummer
	 */
	public void setNummer(int nummer) {
		this.nummer = nummer;
	}

	/**
	 * Setzt den Typ der Stra&szlig;e.
	 * 
	 * @param typ
	 *            Stra&szlig;etyp entsprechend DatK
	 */
	public void setTyp(int typ) {
		this.typ = typ;
	}

	/**
	 * Setzt die Zusatzbezeichnung der Stra&szlig;e.
	 * 
	 * @param zusatz
	 *            Zusatzbezeichnung der Stra&szlig;e
	 */
	public void setZusatz(String zusatz) {
		this.zusatz = zusatz;
	}

}
