/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig.okstra;

import de.bsvrz.kex.bwsin.konfig.ImportException;
import de.bsvrz.kex.bwsin.konfig.Verkehrsrichtung;

/**
 * OKSTRA-Verkehrsrichtung.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: OkstraVerkehrsRichtung.java 10085 2008-07-01 18:35:40Z gieseler $
 * 
 */

public final class OkstraVerkehrsRichtung {
	/**
	 * OKSTRA-Richtungskennung in Stationierungsrichtung.
	 */
	public static final String RICHTUNG_IN_STATRICHTUNG = "R";

	/**
	 * OKSTRA-Richtungskennung gegen Stationierungsrichtung.
	 */
	public static final String RICHTUNG_GEGEN_STATRICHTUNG = "G";

	/**
	 * OKSTRA-Richtungskennung beide Richtungen.
	 */
	public static final String RICHTUNG_BEIDE_RICHTUNGEN = "B";

	/**
	 * Konstante zum Zugriff auf das Attribut
	 * "okstra:gilt_fuer_Verkehrsrichtung.okstra:Verkehrsrichtung_SE.okstra:Kennung".
	 */
	// public static final String RICHTUNG =
	// "okstra:gilt_fuer_Verkehrsrichtung.okstra:Verkehrsrichtung_SE.okstra:Kennung";
	public static final String RICHTUNG = "okstra:Verkehrsrichtung_SE.okstra:Kennung";

	/**
	 * Bestimmt die Verkehrsrichtung aus der OKSTRA-Richtungsangabe.
	 * 
	 * @param okstraFeature
	 *            OKSTRA-Feature
	 * @param fid
	 *            Feature-ID
	 * @return Verkehrsrichtung
	 * @throws ImportException
	 *             wenn die Richtung nicht bestimmt werden kann
	 */
	public static Verkehrsrichtung bestimmeVerkehrsRichtung(
			final OkstraFeature okstraFeature, final String fid)
			throws ImportException {
		Object[] okstrarichtungen = Okstra.getInstance()
				.getSpecialAttributeList(okstraFeature, fid, RICHTUNG);

		if (okstrarichtungen == null || okstrarichtungen.length == 0) {
			throw new ImportException("Die Verkehrsrichtung des Features " + //$NON-NLS-1$
					fid + " kann nicht bestimmt werden (keine Angabe)"); //$NON-NLS-1$
		}

		String okstrarichtung = (String) okstrarichtungen[0];

		if (okstrarichtung
				.equalsIgnoreCase(OkstraVerkehrsRichtung.RICHTUNG_IN_STATRICHTUNG)) {
			return Verkehrsrichtung.RICHTUNG_IN_STATIONIERUNGSRICHTUNG;
		}

		if (okstrarichtung
				.equalsIgnoreCase(OkstraVerkehrsRichtung.RICHTUNG_BEIDE_RICHTUNGEN)) {
			return Verkehrsrichtung.RICHTUNG_BEIDE_RICHTUNGEN;
		}

		if (okstrarichtung
				.equalsIgnoreCase(OkstraVerkehrsRichtung.RICHTUNG_GEGEN_STATRICHTUNG)) {
			return Verkehrsrichtung.RICHTUNG_GEGEN_STATIONIERUNGSRICHTUNG;
		}

		throw new ImportException(
				"Die Verkehrsrichtung des Features " + //$NON-NLS-1$
						fid
						+ " kann nicht bestimmt werden (unbekannte OKSTRA-Richtungskennung '"
						+ okstrarichtung + "')");
	}

	/**
	 * Privater Konstruktor.
	 */
	private OkstraVerkehrsRichtung() {
		/**
		 * damit CheckStyle Ruhe gibt.
		 */
	}
}
