/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig.okstra.parser;

import java.util.HashMap;
import java.util.Map;
import java.util.Stack;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.geotools.xml.handlers.BWSIN_ElementHandlerFactory;
import org.geotools.xml.handlers.DocumentHandler;
import org.geotools.xml.handlers.ElementHandlerFactory;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;

/**
 * XML-Handler WFS-Parser.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: SINXMLHandler.java 10472 2008-07-11 11:22:39Z gieseler $
 * 
 */

public class SINXMLHandler extends DefaultHandler {

	/**
	 * Logger.
	 */
	protected static final Logger LOGGER = getLogger();

	/**
	 * Liefert einen Logger.
	 * 
	 * @return Logger
	 */
	private static Logger getLogger() {
		Logger l = Logger.getLogger("BWSINWFS");
		l.setLevel(Level.WARNING);
		for (Handler h : l.getHandlers()) {
			h.setLevel(Level.WARNING);
		}
		return l;
	}

	/**
	 * Feature-Handler.
	 */
	SINFeatureHandler fh = null;

	/**
	 * Level.
	 */
	// private int level;
	/**
	 * Handler-Stack.
	 */
	private final Stack<DocumentHandler> handlers = new Stack<DocumentHandler>();

	/**
	 * Haupt-Dokument-Handler.
	 */
	private DocumentHandler document = null;

	/**
	 * Element-Factory.
	 */
	private final ElementHandlerFactory ehf = new BWSIN_ElementHandlerFactory(
			LOGGER);
	/**
	 * Feature-Tabelle.
	 */
	Map<String, SINFeatureHandler> fliste = new HashMap<String, SINFeatureHandler>();

	/**
	 * Aktuelles Element.
	 */
	private SINElementHandler current;

	/**
	 * Konstruktor.
	 */
	public SINXMLHandler() {
		super();
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see org.xml.sax.helpers.DefaultHandler#characters(char[], int, int)
	 */
	@Override
	public void characters(char[] ch, int start, int length) {
		// System.out.print("Characters: \"");
		// StringBuffer text = new StringBuffer();
		// text.append(ch, start, length);
		if (current != null) {
			current.addText(ch, start, length);
		}

		// for (int i = start; i < start + length; i++) {
		// switch (ch[i]) {
		// case '\\':
		// System.out.print("\\\\");
		// break;
		// case '"':
		// System.out.print("\\\"");
		// break;
		// case '\n':
		// System.out.print("\\n");
		// break;
		// case '\r':
		// System.out.print("\\r");
		// break;
		// case '\t':
		// System.out.print("\\t");
		// break;
		// default:
		// System.out.print(ch[i]);
		// break;
		// }
		// }
		// System.out.print("\"\n");
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see org.xml.sax.helpers.DefaultHandler#endDocument()
	 */
	@Override
	public void endDocument() {
		// System.out.println("End document");
		document = (handlers.pop());
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see org.xml.sax.helpers.DefaultHandler#endElement(java.lang.String,
	 *      java.lang.String, java.lang.String)
	 */
	@Override
	public void endElement(String uri, String name, String qName) {
		// if ("".equals (uri))
		// System.out.println("End element: " + qName);
		// else
		// System.out.println("End element: {" + uri + "}" + name);

		if (name.equals("featureMember")) {
			fh.buildFeature();
			fliste.put(fh.getName(), fh);
			fh = null;
		} else if (fh != null) {
			try {
				fh.getLast().endElement(uri, name);
			} catch (Exception e) {
				e.printStackTrace();
			}

			// } catch (OperationNotSupportedException e) {
			// e.printStackTrace();
			// } catch (SAXException e) {
			// e.printStackTrace();
			// }
		}
	}

	/**
	 * Gibt die Feature-Tabelle zur&uuml;ck.
	 * 
	 * @return Feature-Tabelle
	 */
	public Map<String, SINFeatureHandler> getFeatures() {
		return fliste;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see org.xml.sax.helpers.DefaultHandler#startDocument()
	 */
	@Override
	public void startDocument() {
		// System.out.println("Start document");
		document = new DocumentHandler(ehf);
		handlers.push(document);
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see org.xml.sax.helpers.DefaultHandler#startElement(java.lang.String,
	 *      java.lang.String, java.lang.String, org.xml.sax.Attributes)
	 */
	@Override
	public void startElement(String uri, String name, String qName,
			Attributes atts) {
		// if ("".equals (uri))
		// System.out.println("Start element: " + qName);
		// else
		// System.out.println("Start element: {" + uri + "}" + name);

		if (name.equalsIgnoreCase("featureMember")) {
			fh = new SINFeatureHandler();
			fh.startElement(uri, name, atts);
			// level = 0;
			// current = fh;
		} else {
			if (fh != null) {
				SINElementHandler seh = new SINElementHandler();
				seh.startElement(uri, name, atts);
				fh.add(seh);
				current = seh;
			}
		}

	}
}
