/*
 *    GeoTools - OpenSource mapping toolkit
 *    http://geotools.org
 *    (C) 2004-2006, GeoTools Project Managment Committee (PMC)
 *    
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 */
package org.geotools.xml.handlers.xsi;

import java.net.URI;
import java.util.logging.Level;

import org.geotools.xml.XSIElementHandler;
import org.geotools.xml.schema.Schema;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotSupportedException;

/**
 * RootHandler purpose.
 * 
 * <p>
 * This is intended to bootstrap the schema parsing
 * </p>
 * 
 * @author dzwiers, Refractions Research, Inc. http://www.refractions.net
 * @author $Author: gysi $ (last modification)
 * @source $URL:
 *         http://svn.geotools.org/geotools/trunk/gt/modules/library/xml/src/main/java/org/geotools/xml/handlers/xsi/RootHandler.java $
 * @version $Id: BWSIN_RootHandler.java 10382 2008-07-09 12:05:48Z gieseler $
 */
public class BWSIN_RootHandler extends RootHandler {
	/** 'root' */
	public final static String LOCALNAME = "root";
	private BWSIN_SchemaHandler schema;
	private final URI uri;

	// /*
	// * should not be called
	// */
	// private BWSIN_RootHandler() {
	// // do nothing
	// }

	/**
	 * Creates a new RootHandler object.
	 * 
	 * @param uri
	 */
	public BWSIN_RootHandler(URI uri) {
		super(uri);
		this.uri = uri;
		schema = new BWSIN_SchemaHandler();
		logger.setLevel(Level.WARNING);
	}

	/**
	 * @see org.geotools.xml.XSIElementHandler#getHandler(java.lang.String,
	 *      java.lang.String)
	 */
	@Override
	public XSIElementHandler getHandler(String namespaceURI, String localName) {
		if (SchemaHandler.LOCALNAME.equalsIgnoreCase(localName)
				&& SchemaHandler.namespaceURI.equalsIgnoreCase(namespaceURI)) {
			if (schema == null) {
				schema = new BWSIN_SchemaHandler();
			}

			return schema;
		}

		logger.warning("Starting schema with " + localName + " element.");

		return null;
	}

	/**
	 * @see org.geotools.xml.XSIElementHandler#getLocalName()
	 */
	@Override
	public String getLocalName() {
		return LOCALNAME;
	}

	/**
	 * <p>
	 * intended to be called after the parse, this generates a Schema object
	 * from the schema which was parsed in.
	 * </p>
	 * 
	 * 
	 * @throws SAXException
	 */
	@Override
	public Schema getSchema() throws SAXException {
		Schema s = schema.compress(uri);

		return s;
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		return LOCALNAME.hashCode() * ((uri == null) ? 1 : uri.hashCode());
	}

	/**
	 * @see org.geotools.xml.XSIElementHandler#startElement(java.lang.String,
	 *      java.lang.String, org.xml.sax.Attributes)
	 */
	@Override
	public void startElement(String namespaceURI, String localName,
			Attributes attr) throws SAXException {
		throw new SAXNotSupportedException(
				"Should never have elements at the root level");
	}

	/**
	 * @see org.xml.sax.ContentHandler#startPrefixMapping(java.lang.String,
	 *      java.lang.String)
	 */
	@Override
	public void startPrefixMapping(String arg0, String arg1) {
		schema.startPrefixMapping(arg0, arg1);
	}

	// /**
	// * @see org.geotools.xml.XSIElementHandler#getHandlerType()
	// */
	// public int getHandlerType() {
	// return DEFAULT;
	// }

	// /**
	// * @see org.geotools.xml.XSIElementHandler#endElement(java.lang.String,
	// * java.lang.String)
	// */
	// public void endElement(String namespaceURI, String localName){
	// // do nothing
	// }
}
